/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __BLDC_PWM_H__
#define __BLDC_PWM_H__
/*!
* @file bldc_pwm.h
*
* @brief This file provides PWM module functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwm_output.h"
#include "pwm_input.h"
#include "bldc_common.h"

/* ============================================  Define  ============================================ */
/* define six step square wave PWM register status */
#if (defined GATE_DRIVER_HIGH_HIGH)

    #define PWM_COMMUTATE(P, N, H)      ((1 << (2 * N + 9)) | ((1 << (2 * N) | 1 << (2 * N + 1)) | (1 << (2 * H) | 1 << (2 * H + 1))))
    #define PWM_FORCE_OFF(X)            ((1 << (2 * X + 9)) + 0x00FF)
    #define PWM_FORCE_ON(X)             ((1 << (2 * X + 8)) + 0x00FF)

    #define ALL_CH_OFF                  0x00FF              /*!< PWM all channels off */
    #define U_MODULATION_V_ON           PWM_COMMUTATE(PWM_U_CHANNEL, PWM_V_CHANNEL, PWM_W_CHANNEL)
    #define U_MODULATION_W_ON           PWM_COMMUTATE(PWM_U_CHANNEL, PWM_W_CHANNEL, PWM_V_CHANNEL)
    #define V_MODULATION_W_ON           PWM_COMMUTATE(PWM_V_CHANNEL, PWM_W_CHANNEL, PWM_U_CHANNEL)
    #define V_MODULATION_U_ON           PWM_COMMUTATE(PWM_V_CHANNEL, PWM_U_CHANNEL, PWM_W_CHANNEL)
    #define W_MODULATION_U_ON           PWM_COMMUTATE(PWM_W_CHANNEL, PWM_U_CHANNEL, PWM_V_CHANNEL)
    #define W_MODULATION_V_ON           PWM_COMMUTATE(PWM_W_CHANNEL, PWM_V_CHANNEL, PWM_U_CHANNEL)
    
    #define PWM_U_ON                    PWM_FORCE_ON(PWM_U_CHANNEL)
    #define PWM_W_OFF                   PWM_FORCE_OFF(PWM_W_CHANNEL)
    #define PWM_V_ON                    PWM_FORCE_ON(PWM_V_CHANNEL)
    #define PWM_U_OFF                   PWM_FORCE_OFF(PWM_U_CHANNEL)
    #define PWM_W_ON                    PWM_FORCE_ON(PWM_W_CHANNEL)
    #define PWM_V_OFF                   PWM_FORCE_OFF(PWM_V_CHANNEL)

#elif (defined GATE_DRIVER_LOW_LOW)

    #define PWM_COMMUTATE(P, N, H)      ((0xFF00 & (0 << (2 * N + 9))) | ((1 << (2 * N) | 1 << (2 * N + 1)) | (1 << (2 * H) | 1 << (2 * H + 1))))
    #define PWM_FORCE_OFF(X)            ((0xFF00 & (0 << (2 * X + 9))) + 0x00FF)
    #define PWM_FORCE_ON(X)             ((0xFF00 & (0 << (2 * X + 8))) + 0x00FF)

    #define ALL_CH_OFF                  0xFFFF              /*!< PWM all channels off */
    #define U_MODULATION_V_ON           PWM_COMMUTATE(PWM_U_CHANNEL, PWM_V_CHANNEL, PWM_W_CHANNEL)
    #define U_MODULATION_W_ON           PWM_COMMUTATE(PWM_U_CHANNEL, PWM_W_CHANNEL, PWM_V_CHANNEL)
    #define V_MODULATION_W_ON           PWM_COMMUTATE(PWM_V_CHANNEL, PWM_W_CHANNEL, PWM_U_CHANNEL)
    #define V_MODULATION_U_ON           PWM_COMMUTATE(PWM_V_CHANNEL, PWM_U_CHANNEL, PWM_W_CHANNEL)
    #define W_MODULATION_U_ON           PWM_COMMUTATE(PWM_W_CHANNEL, PWM_U_CHANNEL, PWM_V_CHANNEL)
    #define W_MODULATION_V_ON           PWM_COMMUTATE(PWM_W_CHANNEL, PWM_V_CHANNEL, PWM_U_CHANNEL)

    #define PWM_U_ON                    PWM_FORCE_ON(PWM_U_CHANNEL)
    #define PWM_W_OFF                   PWM_FORCE_OFF(PWM_W_CHANNEL)
    #define PWM_V_ON                    PWM_FORCE_ON(PWM_V_CHANNEL)
    #define PWM_U_OFF                   PWM_FORCE_OFF(PWM_U_CHANNEL)
    #define PWM_W_ON                    PWM_FORCE_ON(PWM_W_CHANNEL)
    #define PWM_V_OFF                   PWM_FORCE_OFF(PWM_V_CHANNEL)
#else

#endif

/* ===========================================  Typedef  ============================================ */
typedef enum {
    INVALID_VECTOR_MIN = 0,                                 /*!< min invalid ID of six step square wave */
    AH_PWM_BL_ON,                                           /*!< A+B- of six step square wave */
    AH_PWM_CL_ON,                                           /*!< A+C- of six step square wave */
    BH_PWM_CL_ON,                                           /*!< B+C- of six step square wave */
    BH_PWM_AL_ON,                                           /*!< B+A- of six step square wave */
    CH_PWM_AL_ON,                                           /*!< C+A- of six step square wave */
    CH_PWM_BL_ON,                                           /*!< C+B- of six step square wave */
    INVALID_VECTOR_MAX                                      /*!< max invalid ID of six step square wave */
} BLDC_COMMUTATION_VECTOR;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief update six step square wave magnetic vector by current hall status.
*
* @param[in] motorDir: motor run direction
* @param[in] hallStatus: current rotor position hall status
* @return none
*/
void BLDC_Commutate_BaseSector(int16_t motorDir, uint8_t hallStatus);

/*!
* @brief Clear pwm duty and stop bldc motor.
*
* @param[in] none
* @return none
*/
void BLDC_Stop(void);

/*!
* @brief Get PWM module effective duty in 1 times
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] PWMx: PWM Module
* @return pwm effective duty
*/
float Get_EffectiveDuty(BLDC_VARS_CTRL *bldcCtrl, PWM_Type *PWMx);

/*!
* @brief update PWM channel output by six step square wave magnetic vector to driver MOS.
*
* @param[in] PWMx: PWM Module
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] ChangePhaseVector: current magnetic vector in bldc control
* @return none
*/
void MosDriver(PWM_Type *PWMx, BLDC_VARS_CFG *bldcCfg, uint8_t ChangePhaseVector);

/*!
* @brief Update PWM output duty by manual settig or current PID calculate.
*
* @param[in] PWMx: PWM Module
* @param[in] pwmDuty: range: 0 ~ PWMx->MCVR, corresponding to 0 ~ 100% duty cycle
* @return none
*/
void UpDatePwmDuty(PWM_Type *PWMx, uint16_t pwmDuty);

/* ======================================  Functions define  ======================================== */

#endif /* __BLDC_PWM_H__ */

/* =============================================  EOF  ============================================== */
