/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_pwm.c
*
* @brief This file provides PWM module functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_pwm.h"
#include "bldc_keys.h"
#include "bldc_mcu_config.h"
#include "bldc_parameters_define.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Clear pwm duty and stop bldc motor.
*
* @param[in] none
* @return none
*/
void BLDC_Stop(void)
{
    PWM_MODULE->CHOSWCR = ALL_CH_OFF;
    UpDatePwmDuty(PWM_MODULE, 0);

    PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWM_MODULE), true);
}

/*!
* @brief Get PWM module effective duty in 1 times
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] PWMx: PWM Module
* @return pwm effective duty
*/
float Get_EffectiveDuty(BLDC_VARS_CTRL *bldcCtrl, PWM_Type *PWMx)
{
    uint32_t temp_Mcvr;
    uint32_t temp_ChVal;

    temp_Mcvr = PWMx->MCVR;
#if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    temp_ChVal = PWMx->CHANNELS[0].CHnV;
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    bldcCtrl->pwmOutDuty = (float)((temp_Mcvr - temp_ChVal) / (temp_Mcvr * 1.0f));
    #else
    bldcCtrl->pwmOutDuty = (float)(temp_ChVal / (temp_Mcvr * 1.0f));
    #endif
#else
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    temp_ChVal = PWMx->CHANNELS[1].CHnV;
    bldcCtrl->pwmOutDuty = (float)(temp_ChVal / (temp_Mcvr * 1.0f));
    #else
    temp_ChVal = PWMx->CHANNELS[0].CHnV;
    bldcCtrl->pwmOutDuty = (float)(temp_ChVal / (temp_Mcvr * 1.0f));
    #endif
#endif

    /* enlarge 1 times, 0.1234 means 12.34% */
    return bldcCtrl->pwmOutDuty;
}

/*!
* @brief Update PWM output duty by manual settig or current PID calculate.
*
* @param[in] PWMx: PWM Module
* @param[in] pwmDuty: range: 0 ~ PWMx->MCVR, corresponding to 0 ~ 100% duty cycle
* @return none
*/
void UpDatePwmDuty(PWM_Type *PWMx, uint16_t pwmDuty)
{
    if (pwmDuty > (PWMx->MCVR - 1))
    {
        pwmDuty = (PWMx->MCVR - 1);
    }
    else if (pwmDuty < 1)
    {
        pwmDuty = 1;
    }
    else
    {

    }
#if (defined BLDC_AFTER_FLOW_CONTROL)
    if (g_bemfControl.commutateState == 1)
    {
        if (g_bemfControl.changePhaseTime > ((g_bemfControl.afterFlowTime * AFTER_FLOW_ANGLE_NUMERATOR) >> AFTER_FLOW_ANGLE_PSRC))
        {
            switch (g_bemfControl.currentDriverVector)
            {
            case AH_PWM_BL_ON:
            PWMx->CHOSWCR = U_MODULATION_V_ON;
            break;

            case AH_PWM_CL_ON:
            PWMx->CHOSWCR = U_MODULATION_W_ON;
            break;

            case BH_PWM_CL_ON:
            PWMx->CHOSWCR = V_MODULATION_W_ON;
            break;

            case BH_PWM_AL_ON:
            PWMx->CHOSWCR = V_MODULATION_U_ON;
            break;

            case CH_PWM_AL_ON:
            PWMx->CHOSWCR = W_MODULATION_U_ON;
            break;

            case CH_PWM_BL_ON:
            PWMx->CHOSWCR = W_MODULATION_V_ON;
            break;

            default:
            break;
            }
            g_bemfControl.commutateState = 2;
        }
    }
#endif

#if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_U_CHANNEL + 1), (PWMx->MCVR - pwmDuty));
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_V_CHANNEL + 1), (PWMx->MCVR - pwmDuty));
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_W_CHANNEL + 1), (PWMx->MCVR - pwmDuty));
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_U_CHANNEL), (PWMx->MCVR - pwmDuty));
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_V_CHANNEL), (PWMx->MCVR - pwmDuty));
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_W_CHANNEL), (PWMx->MCVR - pwmDuty));
    g_bldcVarsCtrl.matchTriggerValue = PWMx->MCVR;
    #else
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_U_CHANNEL + 1), pwmDuty);
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_V_CHANNEL + 1), pwmDuty);
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_W_CHANNEL + 1), pwmDuty);
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_U_CHANNEL), pwmDuty);
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_V_CHANNEL), pwmDuty);
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_W_CHANNEL), pwmDuty);
    g_bldcVarsCtrl.matchTriggerValue = 1;
    #endif
#else
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_U_CHANNEL + 1), pwmDuty);
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_V_CHANNEL + 1), pwmDuty);
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_W_CHANNEL + 1), pwmDuty);
    #else
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_U_CHANNEL), pwmDuty);
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_V_CHANNEL), pwmDuty);
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_W_CHANNEL), pwmDuty);
    #endif
    g_bldcVarsCtrl.matchTriggerValue = pwmDuty >> 1;
    if (g_bldcVarsCtrl.matchTriggerValue < 1)
    {
        g_bldcVarsCtrl.matchTriggerValue = 1;
    }
#endif
    PWM_DRV_SetChannelCountValue(PWM_INDEX(PWMx), (pwm_channel_type_t)(2 * PWM_T_CHANNEL), g_bldcVarsCtrl.matchTriggerValue);
    PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
}

/*!
* @brief Control afterflow time in bldc sensorless control, reduce afterflow time.
*
* @param[in] PWMx: PWM Module
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] Vector: current magnetic vector in bldc sensorless control
* @return none
*/
void AfterFlow_Control(PWM_Type *PWMx, BLDC_VARS_CFG *bldcCfg, uint8_t Vector)
{
    if (bldcCfg->motorDir == FORWARD_ROTATE)
    {
        switch (Vector)
        {
        case AH_PWM_BL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_U_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = U_MODULATION_V_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        case AH_PWM_CL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_W_OFF;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = U_MODULATION_W_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        case BH_PWM_CL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_V_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = V_MODULATION_W_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        case BH_PWM_AL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_U_OFF;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = V_MODULATION_U_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        case CH_PWM_AL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_W_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = W_MODULATION_U_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        case CH_PWM_BL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_V_OFF;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = W_MODULATION_V_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        default:
        break;
        }
    }
    else
    {
        switch (Vector)
        {
        case AH_PWM_BL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_V_OFF;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = U_MODULATION_V_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        case AH_PWM_CL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_U_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = U_MODULATION_W_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        case BH_PWM_CL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_W_OFF;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = V_MODULATION_W_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        case BH_PWM_AL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_V_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = V_MODULATION_U_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        case CH_PWM_AL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_U_OFF;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = W_MODULATION_U_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        case CH_PWM_BL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWMx->CHOSWCR = PWM_W_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWMx->CHOSWCR = W_MODULATION_V_ON;
        #endif
        PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
        break;

        default:
        break;
        }
    }
}

/*!
* @brief update PWM channel output by six step square wave magnetic vector to driver MOS.
*
* @param[in] PWMx: PWM Module
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] ChangePhaseVector: current magnetic vector in bldc control
* @return none
*/
void MosDriver(PWM_Type *PWMx, BLDC_VARS_CFG *bldcCfg, uint8_t ChangePhaseVector)
{
#if (defined BLDC_AFTER_FLOW_CONTROL)
    AfterFlow_Control(PWMx, bldcCfg, ChangePhaseVector);
#else
    switch (ChangePhaseVector)
    {
    case AH_PWM_BL_ON:
    PWMx->CHOSWCR = U_MODULATION_V_ON;
    break;

    case AH_PWM_CL_ON:
    PWMx->CHOSWCR = U_MODULATION_W_ON;
    break;

    case BH_PWM_CL_ON:
    PWMx->CHOSWCR = V_MODULATION_W_ON;
    break;

    case BH_PWM_AL_ON:
    PWMx->CHOSWCR = V_MODULATION_U_ON;
    break;

    case CH_PWM_AL_ON:
    PWMx->CHOSWCR = W_MODULATION_U_ON;
    break;

    case CH_PWM_BL_ON:
    PWMx->CHOSWCR = W_MODULATION_V_ON;
    break;

    default:
    break;
    }
    PWM_DRV_SetSoftwareTriggerSync(PWM_INDEX(PWMx), true);
#endif
}

/*!
* @brief update six step square wave magnetic vector by current hall status.
*
* @param[in] motorDir: motor run direction
* @param[in] hallStatus: current rotor position hall status
* @return none
*/
void BLDC_Commutate_BaseSector(int16_t motorDir, uint8_t hallStatus)
{
    if (motorDir == FORWARD_ROTATE)
    {
        switch (hallStatus)
        {
        case 1:
            /* B+,C- */
            g_hallControl.currentDriverVector = (uint8_t)BH_PWM_CL_ON;
            break;

        case 2:
            /* A+,B- */
            g_hallControl.currentDriverVector = (uint8_t)AH_PWM_BL_ON;
            break;

        case 3:
            /* A+,C- */
            g_hallControl.currentDriverVector = (uint8_t)AH_PWM_CL_ON;
            break;

        case 4:
            /* C+,A- */
            g_hallControl.currentDriverVector = (uint8_t)CH_PWM_AL_ON;
            break;

        case 5:
            /* B+,A- */
            g_hallControl.currentDriverVector = (uint8_t)BH_PWM_AL_ON;
            break;

        case 6:
            /* C+,B- */
            g_hallControl.currentDriverVector = (uint8_t)CH_PWM_BL_ON;
            break;

        default:
            BLDC_Stop();
            break;
        }
    }
    else
    {
        switch (hallStatus)
        {
        case 1:
            /* C+,B- */
            g_hallControl.currentDriverVector = (uint8_t)CH_PWM_BL_ON;
            break;

        case 2:
            /* B+,A- */
            g_hallControl.currentDriverVector = (uint8_t)BH_PWM_AL_ON;
            break;

        case 3:
            /* C+,A- */
            g_hallControl.currentDriverVector = (uint8_t)CH_PWM_AL_ON;
            break;

        case 4:
            /* A+,C- */
            g_hallControl.currentDriverVector = (uint8_t)AH_PWM_CL_ON;
            break;

        case 5:
            /* A+,B- */
            g_hallControl.currentDriverVector = (uint8_t)AH_PWM_BL_ON;
            break;

        case 6:
            /* B+,C- */
            g_hallControl.currentDriverVector = (uint8_t)BH_PWM_CL_ON;
            break;

        default:
            BLDC_Stop();
            break;
        }
    }
    MosDriver(PWM_MODULE, pBldcVarsCfg, g_hallControl.currentDriverVector);
}

/* =============================================  EOF  ============================================== */
